/**
 * Copyright (c) 2005 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.kepler.scia;


/*-------------------------------------------------------------------------------
  Guilian Wang
  Oct.18, 2004

  File Name:      ViewEditor.java
  Description:    This class serves as an simple editor.
                  It prompts the user with some text and allow the user to edit,
                  return the modified text. It is mainly for users to review
                  the generated view
  -----------------------------------------------------------------------------*/
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


public class ResultViewer extends JFrame {
    // global variables so that they could be set
    private String message;

    // global variables so that they could be set
    private String text;
    JPanel contentPane = new JPanel(new BorderLayout());
    JButton saveButton;
    JButton resetButton;
    JButton cancelButton;
    JTextArea textArea;
    String targetSchemaName;

    // constructor
    public ResultViewer(String tit, String mes, String tex, String t) {
        setTitle(tit);

        // setting the global variable
        message = mes;
        text = tex;
        targetSchemaName = t;

        // position our popup window in the center of the screen
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension d = tk.getScreenSize();

        setLocation((d.width / 2) - 250, (d.height / 2) - 50);
        setSize(500, 500);

        // setting up the panel with a text area      
        textArea = new JTextArea(text, 20, 40);

        JScrollPane areaScrollPane = new JScrollPane(textArea,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        areaScrollPane.setPreferredSize(new Dimension(400, 400));

        //JPanel textPan = new JPanel();
        //textPan.add(textArea);
        JLabel label = new JLabel(message);
        JPanel labPan = new JPanel();
        labPan.add(label);

        /*
        resetButton = new JButton("Reset");
        ActionListener aLreset = new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if (ae.getSource() == resetButton) {
                    try {
                        //restore the original text as view
                        textArea.setText(text);
                        if(SCIA.debug_on) System.err.println("view = \n" + text);
                        PrintWriter pw =
                            new PrintWriter(new FileOutputStream("view.qlt"),
                                            true);

                        //java.util.Date date = new java.util.Date();
                        pw.println(text);
                        pw.flush();
                    } catch (Exception e){
                        if(SCIA.debug_on) System.err.println(" error from EditText resetButton::: " + e);
                    }
                }
            }
        };
        resetButton.addActionListener(aLreset);
        */
        cancelButton = new JButton("Cancel");

        ActionListener aLcancel = new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    if (ae.getSource() == cancelButton) {
                        hide();
                    }
                }
            };

        cancelButton.addActionListener(aLcancel);

        saveButton = new JButton("Save");

        ActionListener aLsave = new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    if (ae.getSource() == saveButton) {
                        try {
                            //save the document as target document
                            if (SCIA.debug_on) {
                                System.err.println("saveButton is pressed");
                            }

                            String editedResult = textArea.getText();

                            if (SCIA.debug_on) {
                                System.err.println("edited result = \n" +
                                    editedResult);
                            }

                            // to get the target document name
                            String targetDoc = "virtual_" + targetSchemaName +
                                ".xml";
                            PrintWriter pw = new PrintWriter(new FileOutputStream(
                                        targetDoc), true);

                            //java.util.Date date = new java.util.Date();
                            pw.println(editedResult);
                            pw.flush();

                            if (SCIA.debug_on) {
                                System.err.println(
                                    "Translated document is saved in" +
                                    " file:" + targetDoc);
                            }
                        } catch (Exception e) {
                            if (SCIA.debug_on) {
                                System.err.println(" error from ResultViewer" +
                                    "saveButton::: " + e);
                            }
                        }
                    }
                }
            };

        saveButton.addActionListener(aLsave);

        JPanel buttonPan = new JPanel();
        buttonPan.add(saveButton);

        //buttonPan.add(resetButton, BorderLayout.CENTER);
        buttonPan.add(cancelButton);

        contentPane.add(buttonPan, BorderLayout.SOUTH);
        contentPane.add(labPan, BorderLayout.NORTH);

        //contentPane.add(textPan, BorderLayout.CENTER);
        contentPane.add(areaScrollPane, BorderLayout.CENTER);

        setContentPane(contentPane);
    }

    // for testing purposes
    public static void main(String[] args) {
        try {
            JFrame f = new ResultViewer("Query Result Editor",
                    "Please check the query result",
                    "saduhgdkwrfgwuirfyiwrrfuwiurewukazueursafywe", "target");
            f.show();
        } catch (Exception e) {
        }
    }
}
