/* CollectionManager for managing new collections created by an actor.
 *
 * Copyright (c) 2005 Natural Diversity Discovery Project.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE NATURAL DIVERSITY DISCOVERY PROJECT BE LIABLE 
 * TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
 * CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND 
 * ITS DOCUMENTATION, EVEN IF THE NATURAL DIVERSITY DISCOVERY PROJECT
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE NATURAL DIVERSITY DISCOVERY PROJECT SPECIFICALLY DISCLAIMS ANY 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NATURAL 
 * DIVERSITY DISCOVERY PROJECT HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

   @ProposedRating Red (tmcphillips@naturaldiversity.org)
   @AcceptedRating Red (tmcphillips@naturaldiversity.org) 
 */
 
package org.nddp;

import ptolemy.kernel.util.IllegalActionException;
 
/** 
  @author Timothy M. McPhillips
*/ 

public final class NewCollectionManager 
		extends CollectionManager {

	public NewCollectionManager(Collection collection, CollectionManager
        	parentCollectionManager, CollectionPort outputPort)
		throws IllegalActionException {	
	
		super(collection, parentCollectionManager, outputPort);
	}	

	///////////////////////////////////////////////////////////////////
	//// public methods for CollectionManager.NewCollectionManager //// 

	public void closeCollection() throws IllegalActionException {
		if ( parentCollectionManager() == null) {
			_sendCollection();
		}
	}

	///////////////////////////////////////////////////////////////////
	////      package protected methods for NewCollectionManager   //// 

	void _sendCollection() throws IllegalActionException {
		
		if ( ! _collectionClosed ) {
			
			// transmit opening delimiter (includes any metadata)
			_sendOpeningDelimiterToken();
			
			// send metadata token if any metadata defined
			_sendNewMetadataTokens();

			// transmit data tokens	
			_sendNewDataTokens();
	
			// transmit child collections
			_sendChildCollections();

			// send any new exception tokens
			_sendNewExceptions();
			
			// transmit closing delimiter
			_sendClosingDelimiterToken();
		}
	}
}