/* Base class for collection actors with one output port and no input port.
 *
 * Copyright (c) 2005 Natural Diversity Discovery Project.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE NATURAL DIVERSITY DISCOVERY PROJECT BE LIABLE 
 * TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
 * CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND 
 * ITS DOCUMENTATION, EVEN IF THE NATURAL DIVERSITY DISCOVERY PROJECT
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE NATURAL DIVERSITY DISCOVERY PROJECT SPECIFICALLY DISCLAIMS ANY 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NATURAL 
 * DIVERSITY DISCOVERY PROJECT HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

   @ProposedRating Red (tmcphillips@naturaldiversity.org)
   @AcceptedRating Red (tmcphillips@naturaldiversity.org) 
 */

package org.nddp.coactors;

import org.nddp.AtomicCoactor;
import org.nddp.CollectionIOPort;

import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;

/**
 * Basic BActor with a single output port.  Used for deriving BActors 
 * that do not have input ports.
 * 
 *  @author Timothy M. McPhillips
 */
public class CollectionSource extends AtomicCoactor {
 
     /** 
     * Constructs an instance of BSource.  The constructor initializes 
     * the output port.
     * 
     * @param container The run-time container of this actor.
     * @param name The name of this actor.
     */  
    public CollectionSource(CompositeEntity container, String name)
        throws IllegalActionException, NameDuplicationException {
        
        // call the superclass constructor
        super(container, name);

        // create the output port
        output = CollectionIOPort.createOutputPort(this, "output");
    }

	///////////////////////////////////////////////////////////////////
	////                         public variables                  ////

    // The output port.
    public CollectionIOPort output;
}
