/* Actor that discards domain objects not matching some criterion.
 *
 * Copyright (c) 2005 Natural Diversity Discovery Project.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE NATURAL DIVERSITY DISCOVERY PROJECT BE LIABLE 
 * TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
 * CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND 
 * ITS DOCUMENTATION, EVEN IF THE NATURAL DIVERSITY DISCOVERY PROJECT
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE NATURAL DIVERSITY DISCOVERY PROJECT SPECIFICALLY DISCLAIMS ANY 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NATURAL 
 * DIVERSITY DISCOVERY PROJECT HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

   @ProposedRating Red (tmcphillips@naturaldiversity.org)
   @AcceptedRating Red (tmcphillips@naturaldiversity.org) 
 */
 
package org.nddp.coactors;

import org.nddp.AtomicCoactor;
import org.nddp.CollectionHandler;
import org.nddp.CollectionManager;
import org.nddp.DataTypes;
import org.nddp.DomainObject;
import org.nddp.util.Parameters;

import ptolemy.data.expr.Parameter;
import ptolemy.data.Token;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

/**
 * @nddp.coactor type="general"
 */
public class DataFilter extends CollectionTransformer {

    public DataFilter(CompositeEntity container, String name)
        throws NameDuplicationException, IllegalActionException  {

        super(container, name);
        
        property = Parameters.stringParameter(this, "property", "");
        value = Parameters.stringParameter(this, "value", "");
        
//        _setDefaultDataDisposition(DISCARD_TOKEN);
    }

	///////////////////////////////////////////////////////////////////
	////                         public variables                  ////

    public Parameter property;
    public Parameter value;

	///////////////////////////////////////////////////////////////////
	////                      protected methods                    //// 
    
    public CollectionHandler.TokenDisposition _handleDomainObject(
        CollectionManager collectionManager, DomainObject object) 
		throws IllegalActionException {
 		
	    String value = DataTypes.getStringProperty(object, _property);
 
      	if (value != null && value.trim().equals(_value)) {        	    
        		return  CollectionHandler.FORWARD_TOKEN;
        	}
        	
	   	return  CollectionHandler.DISCARD_TOKEN;
  	}

    public void _handleParameterChange(Parameter parameter, Token newValue) 
		throws IllegalActionException {
            
        if (parameter == property) {
        
            _property = Parameters.stringValue(newValue);
                      
        } else if (parameter == value) {
        
            _value = Parameters.stringValue(newValue);
                      
        } else {
        
            super._handleParameterChange(parameter, newValue);
        }
    }

	///////////////////////////////////////////////////////////////////
	////                         private variables                 ////

    private String _property;
    private String _value;  
}
