/* An actor for reading SRB files.

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.srb;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.ArrayToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.ObjectToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.UnsignedByteToken;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import edu.sdsc.grid.io.srb.SRBFile;
import edu.sdsc.grid.io.srb.SRBFileSystem;
import edu.sdsc.grid.io.srb.SRBRandomAccessFile;

//////////////////////////////////////////////////////////////////////////
//// SRBReader
/** This actor reads an SRB file.
 *
@author Bing Zhu & Efrat Jaeger
@version $Id: SRBReader.java,v 1.6 2005/11/01 20:39:15 ruland Exp $
@since Ptolemy II 3.0.2
*/
public class SRBReader extends TypedAtomicActor {

 /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */
public SRBReader(CompositeEntity container, String name)
        throws NameDuplicationException, IllegalActionException  {

    super(container, name);

    SRBFileSystem = new TypedIOPort(this, "SRBFileSystem", true, false);
    output = new TypedIOPort(this, "output", false, true);
    remoteFileName = new TypedIOPort(this, "remoteFileName", true, false);
    endOfFile = new TypedIOPort(this, "endOfFile", false, true);

    // Set the type constraint.
    SRBFileSystem.setTypeEquals(BaseType.GENERAL);
    output.setTypeEquals(new ArrayType(BaseType.UNSIGNED_BYTE));
    remoteFileName.setTypeEquals(BaseType.STRING);
    endOfFile.setTypeEquals(BaseType.BOOLEAN);

    _attachText("_iconDescription", "<svg>\n"
            + "<rect x=\"-25\" y=\"-20\" "
            + "width=\"68\" height=\"40\" "
            + "style=\"fill:white\"/>\n"
            + "<polygon points=\"-15,-10 -7,-10 -3,-14 4,-14 8,-10"
            + " 33,-10 33,10, -15,10\" "
            + "style=\"fill:red\"/>\n"
            + "<text x=\"-5\" y=\"7\" "
            + "style=\"font-size:14\">\n"
            + "SRB \n"
            + "</text>\n"
            + "<text x=\"-20\" y=\"19\""
            + "style=\"font-size:11; fill:black; font-family:SansSerif\">"
            + "stream read</text>\n"
            + "</svg>\n");

}

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  /** Connection reference
   */
  public TypedIOPort SRBFileSystem;

  /** Output. Array of bytes.
   */
  public TypedIOPort output;

  public TypedIOPort remoteFileName;

  public TypedIOPort endOfFile;


  ///////////////////////////////////////////////////////////////////
  ////                        public methods                     ////


  /** Accepts an SRBFileSystem reference and a file name and outputs the file content (Array of Bytes).
   */
    public void fire() throws IllegalActionException {
      SRBFile srbFile;
      SRBRandomAccessFile srbRandomAccessFile= null;
      byte[] bytesRead = new byte[20000];;
      int nBytesRead;

      try
      {
          srbFileSystem = (SRBFileSystem)((ObjectToken)SRBFileSystem.get(0)).getValue();
          String _srbFileName = ((StringToken)remoteFileName.get(0)).stringValue();
          srbFile = new SRBFile( srbFileSystem, _srbFileName);
          srbRandomAccessFile = new SRBRandomAccessFile( srbFile, "r");

          nBytesRead = srbRandomAccessFile.read(bytesRead);
          while (nBytesRead > 0) {
            Token _bytes[] = new Token[nBytesRead];

            for (int i = 0; i < nBytesRead; i++) {
              _bytes[i] = new UnsignedByteToken(bytesRead[i]);
            }
            output.send(0, new ArrayToken(_bytes));

            nBytesRead = srbRandomAccessFile.read(bytesRead);
            if (nBytesRead == 0) {
              endOfFile.broadcast(BooleanToken.TRUE);
            } else endOfFile.broadcast(BooleanToken.FALSE);
          }

      } catch (Exception ex) {
        srbFile = null;
        srbRandomAccessFile = null;
        srbFileSystem = SRBUtil.closeConnection(srbFileSystem);
        throw new IllegalActionException(this, ex,
          "Reading exception.");
     }
     srbFile = null;
     srbRandomAccessFile = null;
   }

    /** Post fire the actor. Return false to indicate that the
    * process has finished.
    */
    public boolean postfire() {
      return false;  //FIX ME
    }
    
	/** Disconnect from SRB.
	 */
	public void wrapup() {
		srbFileSystem = SRBUtil.closeConnection(srbFileSystem);
	}

    private SRBFileSystem srbFileSystem = null;

}